import 'package:flutter/foundation.dart';
import 'package:fpdart/fpdart.dart';
import './errors.dart';

@immutable
abstract class ValueObject<F, T> {
  const ValueObject();
  Either<F, T> get value;

  /// Throws [UnexpectedValueError] containing the [ValueFailure]
  T getOrCrash() {
    // id = identity - same as writing (right) => right
    return value.fold((f) => throw UnexpectedValueError(f), id);
  }

  T? getOrNull() {
    return value.fold((f) => null, id);
  }

  Either<F, Unit> get failureOrUnit {
    return value.fold(
      (l) => left(l),
      (r) => right(unit),
    );
  }

  bool isValid() => value.isRight();

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;

    return other is ValueObject<F, T> && other.value == value;
  }

  @override
  int get hashCode => value.hashCode;

  @override
  String toString() => 'Value($value)';
}
