import 'package:fast_immutable_collections/fast_immutable_collections.dart';

/// To create list from [Iterable]
IList<T> iListOf<T>(Iterable<T> list) =>
    <T>[].lock.also((it) => it.addAll(list));

/// To create list from [Iterable]
List<T> listOf<T>(Iterable<T> list) => <T>[].also((it) => it.addAll(list));

/// Executes the given function [action] specified number of [times].
repeat(int times, void Function(int int) action) {
  for (int i = 0; i < times; i++) {
    action(i);
  }
}

/// Calls the specified function [operation] with `this` value as its receiver and returns its result.
ReturnType run<ReturnType>(ReturnType Function() operation) {
  return operation();
}

extension ScopeFunctionsForObject<T extends Object> on T {
  /// Calls the specified function [operation] with `this` value as its argument and returns its result.
  ReturnType let<ReturnType>(ReturnType Function(T self) operationFor) {
    return operationFor(this);
  }

  /// Calls the specified function [operation] with `this` value as its argument and returns `this` value.
  T also(void Function(T self) operationFor) {
    operationFor(this);
    return this;
  }
}
